/**
 * API调用返回实体类型
 */
class ApiReply {

    static EMPTY_RESULT = new ApiReply(0, "OK");

    /**
     *
     * @param {number} errCode
     * @param {string} message
     * @param {*[]?} data
     * @param {number?} count
     */
    constructor(errCode, message, data, count) {

        /**
         *
         * @type {number} 错误码
         */
        this.errCode = errCode;

        /**
         *
         * @type {string} 错误信息
         */
        this.message = message;

        /**
         *
         * @type {*[]} 返回数据数组
         */
        this.data = data;

        /**
         *
         * @type {number} 查询结果的记录数（分页前）
         */
        this.count = count;
    }
}

class GnssError extends Error {

}


// 流媒体状态通知结构
class StrmNotif {

    //////////////////////////
    // 通知类型常量 (act)

    /**
     * 通知类型：已下发指令
     *
     * @returns {string}
     * @constructor
     */
    static get ACT__cmdSent() {
        return 'sent';
    }

    /**
     * 通知类型：指令失败
     *
     * @returns {string}
     * @constructor
     */
    static get ACT__cmdFailed() {
        return 'failed';
    }

    /**
     * 通知类型：媒体已准备好
     * @returns {string}
     * @constructor
     */
    static get ACT__strmReady() {
        return 'ready';
    }

    /**
     * 通知类型：流请求关闭
     *
     * @returns {string}
     * @constructor
     */
    static get ACT__strmClosed() {
        return 'closed';
    }

    /**
     * 通知类型：终端流已经识别
     * @returns {string}
     * @constructor
     */
    static get ACT__strmIdentified() {
        return 'id';
    }

    //////////////////////////
    // 媒体类型

    /**
     * 媒体类型： 实时播放
     *
     * @returns {string}
     * @constructor
     */
    static get TYP__live() {
        return 'live';
    }

    /**
     * 媒体类型：远程回放
     *
     * @returns {string}
     * @constructor
     */
    static get TYP__replay() {
        return 'replay';
    }


    //////////////////////////
    // 流关闭原因

    /**
     * 流关闭原因：客户端请求关闭
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__clientReq() {
        return 1;
    }

    /**
     * 流关闭原因： 终端关闭媒体流
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__termStrmClose() {
        return 2;
    }

    /**
     * 流关闭原因： 连接到媒体服务失败
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__connectStrmServerFailed() {
        return 3;
    }

    /**
     * 流关闭原因： 播放失败（由于编解码错误等原因）
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__exceptionCaught() {
        return 4;
    }

    /**
     * 流关闭原因： 客户端保持超时
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__keepTimeout() {
        return 5;
    }

    /**
     * 流关闭原因：终端媒体数据链路超时
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__termDataTimeout() {
        return 6;
    }

    /**
     * 流关闭原因： 终端媒体格式不能识别或不支持
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__badStrmFormat() {
        return 7;
    }

    /**
     * 流关闭原因：终端不在线或已经下线
     *
     * @returns {number}
     * @constructor
     */
    static get CLOSE_CAUSE__noConnection() {
        return 8;
    }

    /**
     * 流关闭原因：终端指令失败
     *
     * @return {number}
     * @constructor
     */
    static get CLOSE_CAUSE__termCmdFailed() {
        return 9;
    }

    /**
     * 取流关闭原因的文字描述
     *
     * @param closeCause CLOSE_CAUSE__xxxx 系统常量之一
     * @return {string}
     */
    static closeReasonOf(closeCause) {
        switch (closeCause) {
            case StrmNotif.CLOSE_CAUSE__clientReq:
                return '客户端关闭';

            case StrmNotif.CLOSE_CAUSE__termStrmClose:
                return '终端媒体连接断开';

            case StrmNotif.CLOSE_CAUSE__connectStrmServerFailed:
                return '无法连接到中间媒体服务或源媒体服务';

            case StrmNotif.CLOSE_CAUSE__exceptionCaught:
                return '媒体服务内部错误';

            case StrmNotif.CLOSE_CAUSE__keepTimeout:
                return '客户端保持超时';

            case StrmNotif.CLOSE_CAUSE__termDataTimeout:
                return '终端未能在一定时间内推流';

            case StrmNotif.CLOSE_CAUSE__badStrmFormat:
                return '媒体格式错误';

            case StrmNotif.CLOSE_CAUSE__noConnection:
                return '终端不在线或已经下线';

            case StrmNotif.CLOSE_CAUSE__termCmdFailed:
                return '终端指令失败';

            default:
                return '?';
        }
    }


    constructor() {
        this.act = '';
        this.reqId = '';
        this.typ = '';
        /**
         *
         * @type {string | undefined}
         */
        this.servId = undefined;
        this.simNo = '';
        this.chan = 0;
        /**
         *
         * @type {number | undefined}
         */
        this.codeStrm = undefined;

        /**
         * Only valid when `act` == `ready`
         *
         * @type {string | undefined}
         */
        this.mediaTyp = undefined;

        /**
         * Only valid when `act` == `ready`
         *
         * @type {string | undefined}
         */
        this.playUrl = undefined;

        /**
         * This stomp websocket url used to push audio to server.
         *
         * @type {string | undefined}
         * @deprecated Use `taUrl` instead.
         */
        this.wsUrl = undefined;

        /**
         * This binary websocket url used to push audio to server.
         *
         * @type {string | undefined}
         */
        this.taUrl = undefined;

        /**
         *
         * @type {number | undefined}
         */
        this.closeCause = undefined;

        /**
         *
         * @type {string | undefined}
         */
        this.closeReason = undefined;
    }
}



class GnssApi {

    /**
     * 默认保持定时器的时间间隔，单位：毫秒
     * @return {number}
     */
    static get defaultKeepAliveIntervalMillis() {
        return 15_000;
    }


    /**
     * 客户端协议(媒体流格式)：FLV
     *
     * @return {number}
     * @constructor
     */
    static get PROTO__FLV() {
        return 0;
    }

    /**
     * 客户端协议(媒体流格式)：HLS
     *
     * @return {number}
     * @constructor
     */
    static get PROTO__HLS() {
        return 1;
    }

    /**
     * 打开实时播放数据类型：音视频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_DATA_TYPE__AV() {
        return 0;
    }

    /**
     * 打开实时播放数据类型：仅视频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_DATA_TYPE__VIDEO_ONLY() {
        return 1;
    }

    /**
     * 打开实时播放数据类型：对讲
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_DATA_TYPE__TALK() {
        return 2;
    }

    /**
     * 打开实时播放数据类型：监听
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_DATA_TYPE__LISTEN() {
        return 3;
    }

    /**
     * 打开实时播放数据类型：广播
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_DATA_TYPE__BROADCAST() {
        return 4;
    }

    /**
     * 打开实时播放码流：主码流
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_CODE_STREAM__PRIMARY() {
        return 0;
    }

    /**
     * 打开实时播放码流：子码流
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_LIVE_CODE_STREAM__SUB() {
        return 1;
    }

    /**
     * 实时播放控制：关闭音视频
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CTRL__CLOSE_AV() {
        return 0;
    }

    /**
     * 实时播放控制：切换码流
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CTRL__SWITCH_CODE_STREAM() {
        return 1;
    }

    /**
     * 实时播放控制：暂停
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CTRL__PAUSE() {
        return 2;
    }

    /**
     * 实时播放控制：从暂停中恢复
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CTRL__RESUME() {
        return 3;
    }

    /**
     * 实时播放关闭动作：关闭音视频
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CLOSE_ACTION__CLOSE_AV() {
        return 0;
    }

    /**
     * 实时播放关闭动作：关闭音频
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CLOSE_ACTION__CLOSE_AUDIO() {
        return 0;
    }

    /**
     * 实时播放关闭动作：关闭视频
     *
     * @return {number}
     * @constructor
     */
    static get LIVE_CLOSE_ACTION__CLOSE_VIDEO() {
        return 0;
    }

    /**
     * 远程回放模式：正常回放
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MODE__NORMAL() {
        return 0;
    }

    /**
     * 远程回放模式：快进回放
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MODE__FAST_FORWARD() {
        return 1;
    }

    /**
     * 远程回放模式：关键帧快退回放
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MODE__FAST_BACKWARD() {
        return 2;
    }

    /**
     * 远程回放模式：关键帧回放
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MODE__KEY_FRAME() {
        return 3;
    }

    /**
     * 远程回放模式：单帧上传
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MODE__SINGLE_FRAME() {
        return 4;
    }

    /**
     * 远程回放媒体类型：音视频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MEDIA_TYPE__AUDIO_AND_VIDEO() {
        return 0;
    }

    /**
     * 远程回放媒体类型：仅音频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MEDIA_TYPE__AUDIO() {
        return 1;
    }

    /**
     * 远程回放媒体类型：仅视频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MEDIA_TYPE__VIDEO() {
        return 2;
    }

    /**
     * 远程回放媒体类型：音频或视频
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_MEDIA_TYPE__AUDIO_OR_VIDEO() {
        return 3;
    }

    /**
     * 远程回话控制：正常回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__NORMAL() {
        return 0;
    }

    /**
     * 远程回话控制：从暂停中恢复回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__RESUME() {
        return 0;
    }

    /**
     * 远程回话控制：暂停回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__PAUSE() {
        return 1;
    }

    /**
     * 远程回话控制：结束回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__CLOSE() {
        return 2;
    }

    /**
     * 远程回话控制：快进回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__FAST_FORWARD() {
        return 3;
    }

    /**
     * 远程回话控制：关键帧快退回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__KEY_FRAME_BACKWARD() {
        return 4;
    }

    /**
     * 远程回话控制：拖动回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__SEEK() {
        return 5;
    }

    /**
     * 远程回话控制：关键帧回放
     *
     * @return {number}
     * @constructor
     */
    static get REPLAY_CTRL__KEY_FRAME_PLAY() {
        return 6;
    }

    /**
     * 远程回放快进/快退倍速：1倍速
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_FACTOR__1X() {
        return 1;
    }

    /**
     * 远程回放快进/快退倍速：2倍速
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_FACTOR__2X() {
        return 2;
    }

    /**
     * 远程回放快进/快退倍速：4倍速
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_FACTOR__4X() {
        return 3;
    }

    /**
     * 远程回放快进/快退倍速：8倍速
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_FACTOR__8X() {
        return 4;
    }

    /**
     * 远程回放快进/快退倍速：16倍速
     *
     * @return {number}
     * @constructor
     */
    static get OPEN_REPLAY_FACTOR__16X() {
        return 5;
    }

    static replaySpdFactorText(factorId) {
        switch (factorId) {
            case 1:
                return "1X";

            case 2:
                return "2X";

            case 3:
                return "4X";

            case 4:
                return "8X";

            case 5:
                return "16X";

            default:
                return "?";
        }
    }

    static get EVENT_ON_API_ERROR() {
        return "onApiError";
    }

}

